"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: ksp-servicestelle@sgdnord.rlp.de
Created on: 18.08.22

"""
from django.http import HttpRequest
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from compensation.filters.compensation import CompensationTableFilter
from compensation.models import Compensation
from konova.utils.message_templates import DATA_IS_UNCHECKED, DATA_CHECKED_ON_TEMPLATE, DATA_CHECKED_PREVIOUSLY_TEMPLATE
from konova.utils.tables import BaseTable, TableRenderMixin, TableOrderMixin
import django_tables2 as tables


class CompensationTable(BaseTable, TableRenderMixin, TableOrderMixin):
    id = tables.Column(
        verbose_name=_("Identifier"),
        orderable=True,
        accessor="identifier",
    )
    t = tables.Column(
        verbose_name=_("Title"),
        orderable=True,
        accessor="title",
    )
    d = tables.Column(
        verbose_name=_("Parcel gmrkng"),
        orderable=False,
        accessor="geometry",
    )
    c = tables.Column(
        verbose_name=_("Checked"),
        orderable=True,
        empty_values=[],
        accessor="intervention__checked",
    )
    r = tables.Column(
        verbose_name=_("Recorded"),
        orderable=True,
        empty_values=[],
        accessor="intervention__recorded",
    )
    e = tables.Column(
        verbose_name=_("Editable"),
        orderable=True,
        empty_values=[],
        accessor="intervention__users",
    )
    lm = tables.Column(
        verbose_name=_("Last edit"),
        orderable=True,
        accessor="modified__timestamp",
    )

    class Meta(BaseTable.Meta):
        template_name = "django_tables2/bootstrap4.html"

    def __init__(self, request: HttpRequest, *args, **kwargs):
        self.title = _("Compensations")
        self.add_new_url = reverse("compensation:new")
        qs = kwargs.get("queryset", None)
        self.filter = CompensationTableFilter(
            user=request.user,
            data=request.GET,
            queryset=qs,
        )
        kwargs["queryset"] = self.filter.qs
        super().__init__(request, *args, **kwargs)

    def render_id(self, value, record: Compensation):
        """ Renders the id column for a compensation

        Args:
            value (str): The identifier value
            record (Compensation): The compensation record

        Returns:

        """
        context = {
            "tooltip": _("Open {}").format(_("Intervention")),
            "content": value,
            "url": reverse("compensation:detail", args=(record.id,)),
            "has_revocations": record.intervention.legal.revocations.exists()
        }
        html = render_to_string(
            "table/revocation_warning_col.html",
            context
        )
        return html

    def render_c(self, value, record: Compensation):
        """ Renders the checked column for a compensation

        checked is set by the main object Intervention

        Args:
            value (str): The identifier value
            record (Compensation): The compensation record

        Returns:

        """
        html = ""
        checked = value is not None
        tooltip = DATA_IS_UNCHECKED
        previously_checked = record.intervention.get_last_checked_action()
        if checked:
            checked_on = value.get_timestamp_str_formatted()
            tooltip = DATA_CHECKED_ON_TEMPLATE.format(checked_on, record.intervention.checked.user)
        html += self.render_checked_star(
            tooltip=tooltip,
            icn_filled=checked,
        )
        if previously_checked and not checked:
            checked_on = previously_checked.get_timestamp_str_formatted()
            tooltip = DATA_CHECKED_PREVIOUSLY_TEMPLATE.format(checked_on, previously_checked.user)
            html += self.render_previously_checked_star(
                tooltip=tooltip,
            )
        return format_html(html)

    def render_r(self, value, record: Compensation):
        """ Renders the registered column for a compensation

        Args:
            value (str): The identifier value
            record (Compensation): The compensation record

        Returns:

        """
        html = ""
        recorded = value is not None
        tooltip = _("Not recorded yet")
        if recorded:
            on = value.get_timestamp_str_formatted()
            tooltip = _("Recorded on {} by {}").format(on, record.intervention.recorded.user)
        html += self.render_bookmark(
            tooltip=tooltip,
            icn_filled=recorded,
        )
        return format_html(html)

