"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: michel.peltriaux@sgdnord.rlp.de
Created on: 09.02.22

"""
from django.urls import reverse
from django.test.client import Client

from compensation.models import Payment
from konova.settings import DEFAULT_GROUP
from konova.tests.test_views import BaseViewTestCase


class PaymentViewTestCase(BaseViewTestCase):

    @classmethod
    def setUpTestData(cls) -> None:
        super().setUpTestData()

    def setUp(self) -> None:
        super().setUp()
        self.payment = Payment.objects.get_or_create(
            intervention=self.intervention,
            amount=1,
            due_on="2020-01-01",
            comment="Testcomment"
        )[0]

        self.new_url = reverse("compensation:pay:new", args=(self.intervention.id,))
        self.edit_url = reverse("compensation:pay:edit", args=(self.intervention.id, self.payment.id))
        self.remove_url = reverse("compensation:pay:remove", args=(self.intervention.id, self.payment.id))

    def test_anonymous_user(self):
        """ Check correct status code for all requests

        Assumption: User not logged in

        Returns:

        """
        client = Client()

        success_urls = [
        ]
        fail_urls = [
            self.new_url,
            self.edit_url,
            self.remove_url,
        ]

        self.assert_url_success(client, success_urls)
        self.assert_url_fail(client, fail_urls)

    def test_logged_in_no_groups_shared(self):
        """ Check correct status code for all requests

        Assumption: User logged in and has no groups and data is shared

        Returns:

        """
        client = Client()
        client.login(username=self.superuser.username, password=self.superuser_pw)
        self.superuser.groups.set([])
        self.intervention.share_with_user_list([self.superuser])

        # Since the user has no groups, it does not matter that data has been shared. There SHOULD not be any difference
        # to a user without access, since the important permissions are missing
        success_urls = [
        ]
        fail_urls = [
            self.new_url,
            self.edit_url,
            self.remove_url,
        ]

        self.assert_url_success(client, success_urls)
        self.assert_url_fail(client, fail_urls)

    def test_logged_in_no_groups_unshared(self):
        """ Check correct status code for all requests

        Assumption: User logged in and has no groups and data is not shared

        Returns:

        """
        client = Client()
        client.login(username=self.superuser.username, password=self.superuser_pw)
        self.superuser.groups.set([])
        # Sharing is inherited by base intervention for compensation. Therefore configure the interventions share state
        self.intervention.share_with_user_list([])

        # Since the user has no groups, it does not matter that data is unshared. There SHOULD not be any difference
        # to a user having shared access, since all important permissions are missing
        success_urls = [
        ]
        fail_urls = [
            self.new_url,
            self.edit_url,
            self.remove_url,
        ]

        self.assert_url_success(client, success_urls)
        self.assert_url_fail(client, fail_urls)

    def test_logged_in_default_group_shared(self):
        """ Check correct status code for all requests

        Assumption: User logged in, is default group member and data is shared
        --> Default group necessary since all base functionalities depend on this group membership

        Returns:

        """
        client = Client()
        client.login(username=self.superuser.username, password=self.superuser_pw)
        group = self.groups.get(name=DEFAULT_GROUP)
        self.superuser.groups.set([group])
        # Sharing is inherited by base intervention for compensation. Therefore configure the interventions share state
        self.intervention.share_with_user_list([self.superuser])

        success_urls = [
            self.new_url,
            self.edit_url,
            self.remove_url,
        ]
        self.assert_url_success(client, success_urls)

    def test_logged_in_default_group_unshared(self):
        """ Check correct status code for all requests

        Assumption: User logged in, is default group member and data is NOT shared
        --> Default group necessary since all base functionalities depend on this group membership

        Returns:

        """
        client = Client()
        client.login(username=self.superuser.username, password=self.superuser_pw)
        group = self.groups.get(name=DEFAULT_GROUP)
        self.superuser.groups.set([group])
        # Sharing is inherited by base intervention for compensation. Therefore configure the interventions share state
        self.intervention.share_with_user_list([])

        success_urls = [
        ]
        fail_urls = [
            self.new_url,
            self.edit_url,
            self.remove_url,
        ]
        self.assert_url_fail(client, fail_urls)
        self.assert_url_success(client, success_urls)
