"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: michel.peltriaux@sgdnord.rlp.de
Created on: 09.02.22

"""
from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse

from compensation.models import Payment
from konova.tests.test_views import BaseWorkflowTestCase
from user.models import UserAction


class PaymentWorkflowTestCase(BaseWorkflowTestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def setUp(self) -> None:
        super().setUp()
        # Give the user shared access to the dummy intervention
        self.intervention.share_with_user(self.superuser)

        self.payment = Payment.objects.get_or_create(
            intervention=self.intervention,
            amount=1,
            due_on="2020-01-01",
            comment="Testcomment"
        )[0]

    def test_new(self):
        """ Test the creation of a payment

        Returns:

        """
        # Prepare url and form data to be posted
        new_url = reverse("compensation:pay:new", args=(self.intervention.id,))
        test_amount = 12345
        test_due_on = "1970-01-01"
        test_comment = self.create_dummy_string()
        post_data = {
            "amount": test_amount,
            "due": test_due_on,
            "comment": test_comment,
        }
        pre_creation_intervention_log_count = self.intervention.log.count()
        num_payments = self.intervention.payments.count()

        self.client_user.post(new_url, post_data)

        self.intervention.refresh_from_db()

        self.assertEqual(num_payments + 1, self.intervention.payments.count())
        new_payment = self.intervention.payments.get(amount=test_amount)
        self.assertEqual(new_payment.amount, test_amount)
        self.assertEqual(str(new_payment.due_on), test_due_on)
        self.assertEqual(new_payment.comment, test_comment)

        # Expect logs to be set
        self.assertEqual(pre_creation_intervention_log_count + 1, self.intervention.log.count())
        self.assertEqual(self.intervention.log.first().action, UserAction.EDITED)

    def test_edit(self):
        """ Test edit of a payment

        Returns:

        """
        # Prepare url and form data to be posted
        new_url = reverse("compensation:pay:edit", args=(self.intervention.id, self.payment.id))
        test_amount = self.payment.amount * 2
        test_due_on = "1970-01-01"
        test_comment = self.create_dummy_string()
        post_data = {
            "amount": test_amount,
            "due": test_due_on,
            "comment": test_comment,
        }
        pre_edit_intervention_log_count = self.intervention.log.count()
        num_payments = self.intervention.payments.count()

        self.client_user.post(new_url, post_data)

        self.intervention.refresh_from_db()
        self.payment.refresh_from_db()

        self.assertEqual(num_payments, self.intervention.payments.count())
        self.assertEqual(self.payment.amount, test_amount)
        self.assertEqual(str(self.payment.due_on), test_due_on)
        self.assertEqual(self.payment.comment, test_comment)

        # Expect logs to be set
        self.assertEqual(pre_edit_intervention_log_count + 1, self.intervention.log.count())
        self.assertEqual(self.intervention.log.first().action, UserAction.EDITED)

    def test_remove(self):
        """ Test remove of a payment

        Returns:

        """
        # Prepare url and form data to be posted
        new_url = reverse("compensation:pay:remove", args=(self.intervention.id, self.payment.id))
        post_data = {
            "confirm": True,
        }
        pre_remove_intervention_log_count = self.intervention.log.count()
        num_payments = self.intervention.payments.count()

        self.client_user.post(new_url, post_data)

        self.intervention.refresh_from_db()
        try:
            self.payment.refresh_from_db()
            self.fail(msg="Payment still exists after delete")
        except ObjectDoesNotExist:
            pass

        self.assertEqual(num_payments - 1, self.intervention.payments.count())

        # Expect logs to be set
        self.assertEqual(pre_remove_intervention_log_count + 1, self.intervention.log.count())
        self.assertEqual(self.intervention.log.first().action, UserAction.EDITED)

